@@ -1,5 +1,7 @@
 from .__version__ import __version__
 from .base import FAAPI
+from .exceptions import *
 from .journal import Journal
 from .submission import Submission
 from .submission import SubmissionPartial
+from .user import User
