@@ -10,10 +10,11 @@ from cfscrape import CloudflareScraper
 from cfscrape import create_scraper
 from requests import Response
 from requests import get as get_raw
+from urllib3.exceptions import IncompleteRead
 
 from .__version__ import __version__
 
-root = "https://www.furaffinity.net"
+root: str = "https://www.furaffinity.net"
 user_agent: str = f"faapi/{__version__} Python/{python_version()} {(u := uname()).system}/{u.release}"
 
 
@@ -28,7 +29,7 @@ def ping():
 def make_session(cookies: List[dict]) -> Optional[CloudflareScraper]:
     ping()
 
-    cookies = [
+    cookies: List[Dict[str, str]] = [
         {"name": cookie["name"], "value": cookie["value"]}
         for cookie in cookies
         if "name" in cookie and "value" in cookie
@@ -43,16 +44,15 @@ def make_session(cookies: List[dict]) -> Optional[CloudflareScraper]:
 
 
 def get_robots() -> Dict[str, List[str]]:
-    res = get_raw(join_url(root, "robots.txt"), headers={"User-Agent": user_agent})
+    res: Response = get_raw(join_url(root, "robots.txt"), headers={"User-Agent": user_agent})
 
-    if not res.ok:
-        return {}
+    res.raise_for_status()
 
     robot: Dict[str, Union[int, str, List[str]]] = {}
 
     for elem in filter(lambda r: r not in ("", "#"), res.text.split("\n")):
         key, value = elem.split(":", 1)
-        robot[key] = robot.get(key, []) + [value.strip()]
+        robot[key] = [*robot.get(key, []), value.strip()]
 
     return robot
 
@@ -64,15 +64,16 @@ def get(session: CloudflareScraper, path: str, **params) -> Response:
 def get_binary_raw(session: CloudflareScraper, url: str, speed: Union[int, float] = 100) -> Optional[bytes]:
     assert isinstance(speed, int) or isinstance(speed, float)
 
-    file_stream = session.get(url, stream=True, headers={"User-Agent": user_agent})
+    file_stream: Response = session.get(url, stream=True, headers={"User-Agent": user_agent})
 
-    if not file_stream.ok:
-        file_stream.close()
-        return None
+    file_stream.raise_for_status()
 
-    file_binary = bytes()
+    file_binary: bytes = bytes()
     for chunk in file_stream.iter_content(chunk_size=1024):
         file_binary += chunk
         sleep(1 / speed) if speed > 0 else None
 
+    if (length := int(file_stream.headers.get("Content-Length", 0))) > 0 and length != len(file_binary):
+        raise IncompleteRead(l := len(file_binary), length - l)
+
     return file_binary
