@@ -5,14 +5,13 @@ from typing import Union
 from bs4 import BeautifulSoup
 from bs4.element import Tag
 
-from faapi.parse import ParsingError
-from faapi.parse import check_page
-from faapi.parse import parse_journal_page
-from faapi.parse import parse_journal_section
+from .parse import check_page_raise
+from .parse import parse_journal_page
+from .parse import parse_journal_section
 
 
 class Journal:
-    def __init__(self, journal_item: Optional[Union[Tag, BeautifulSoup]] = None):
+    def __init__(self, journal_item: Union[Tag, BeautifulSoup] = None):
         assert journal_item is None or isinstance(journal_item, BeautifulSoup) or isinstance(journal_item, Tag)
 
         self.journal_item: Optional[Union[Tag, BeautifulSoup]] = journal_item
@@ -38,14 +37,15 @@ class Journal:
     def __str__(self):
         return f"{self.id} {self.author} {self.title}"
 
-    def parse(self):
+    def parse(self, journal_item: Union[Tag, BeautifulSoup] = None):
+        assert journal_item is None or isinstance(journal_item, BeautifulSoup) or isinstance(journal_item, Tag)
+        self.journal_item = journal_item if journal_item is not None else self.journal_item
         if self.journal_item is None:
             return
 
         parsed: Dict[str, Union[int, str]]
         if isinstance(self.journal_item, BeautifulSoup):
-            if (err := check_page(self.journal_item)) != 0:
-                raise ParsingError(f"Error: journal page error {err}")
+            check_page_raise(self.journal_item)
             parsed = parse_journal_page(self.journal_item)
         else:
             parsed = parse_journal_section(self.journal_item)
