@@ -1,17 +1,23 @@
+from re import Pattern
+from re import compile as re_compile
 from re import match
-from re import search
 from re import sub
 from typing import Dict
 from typing import List
+from typing import Tuple
 from typing import Union
 
 from bs4 import BeautifulSoup
 from bs4.element import Tag
 from dateutil.parser import parse as parse_date
 
+from .exceptions import DisabledAccount
+from .exceptions import NoTitle
+from .exceptions import NonePage
+from .exceptions import NoticeMessage
+from .exceptions import ServerError
 
-class ParsingError(Exception):
-    pass
+mentions_regexp: Pattern = re_compile(r"^(?:(?:https?://)?(?:www.)?furaffinity.net)?/user/(.+)$")
 
 
 def parse_page(text: str) -> BeautifulSoup:
@@ -42,17 +48,40 @@ def check_page(page: BeautifulSoup) -> int:
     return 0
 
 
+def check_page_raise(page: BeautifulSoup):
+    if (check := check_page(page)) == 1:
+        raise NonePage
+    elif check == 2:
+        raise NoTitle
+    elif check == 3:
+        raise DisabledAccount
+    elif check == 4:
+        raise ServerError
+    elif check == 5:
+        raise NoticeMessage
+
+
+def parse_mentions(tag: Tag) -> List[str]:
+    return sorted(filter(bool, set(
+        sub(r"[^a-z0-9.~-]", "", m.group(1).lower())
+        for a in tag.findAll("a")
+        if (m := match(mentions_regexp, a.attrs.get("href"))) is not None
+    )))
+
+
 def parse_journal_section(section_tag: Tag) -> Dict[str, Union[int, str]]:
     id_: int = int(section_tag.attrs["id"][4:])
     title: str = section_tag.find("h2").text.strip()
     date: str = parse_date(section_tag.find("span", class_="popup_date")["title"].strip()).strftime("%Y-%m-%d")
-    content: str = "".join(map(str, section_tag.find("div", class_="journal-body").children))
+    content: str = "".join(map(str, (tag_content := section_tag.find("div", class_="journal-body")).children))
+    mentions: List[str] = parse_mentions(tag_content)
 
     return {
         "id": id_,
         "title": title,
         "date": date,
         "content": content,
+        "mentions": mentions,
     }
 
 
@@ -63,13 +92,14 @@ def parse_journal_page(journal_page: BeautifulSoup) -> Dict[str, Union[int, str]
     tag_title: Tag = journal_page.find("h2", class_="journal-title")
     tag_author: Tag = journal_page.find("a", class_="current")
     tag_date: Tag = journal_page.find("span", class_="popup_date")
-    tag_body: Tag = journal_page.find("div", class_="journal-content")
+    tag_content: Tag = journal_page.find("div", class_="journal-content")
 
     id_: int = int(tag_id["content"].strip("/").split("/")[-1])
     title: str = tag_title.text.strip()
     author: str = tag_author["href"].strip("/").split("/")[-1]
     date: str = parse_date(tag_date["title"].strip()).strftime("%Y-%m-%d")
-    content: str = "".join(map(str, tag_body.children)).strip()
+    content: str = "".join(map(str, tag_content.children)).strip()
+    mentions: List[str] = parse_mentions(tag_content)
 
     return {
         "id": id_,
@@ -77,6 +107,7 @@ def parse_journal_page(journal_page: BeautifulSoup) -> Dict[str, Union[int, str]
         "title": title,
         "date": date,
         "content": content,
+        "mentions": mentions,
     }
 
 
@@ -114,6 +145,10 @@ def parse_submission_page(sub_page: BeautifulSoup) -> Dict[str, Union[int, str,
     tag_species: Tag = tag_info[2].find("span")
     tag_gender: Tag = tag_info[3].find("span")
     tag_description: Tag = sub_page.find("div", class_="submission-description")
+    tag_folder: Tag = next(filter(
+        lambda a: a["href"].startswith("/scraps/") or a["href"].startswith("/gallery/"),
+        sub_page.findAll("a", class_="button")
+    ))
     tag_file_url: Tag = sub_page.find("div", class_="download").find("a")
 
     id_: int = int(tag_id["content"].strip("/").split("/")[-1])
@@ -130,11 +165,8 @@ def parse_submission_page(sub_page: BeautifulSoup) -> Dict[str, Union[int, str,
     gender: str = tag_gender.text.strip()
     rating: str = tag_rating.text.strip()
     description: str = "".join(map(str, tag_description.children)).strip()
-    mentions: List[str] = sorted(set(
-        sub(r"[^a-z0-9.~-]", "", m.group(1).lower())
-        for a in tag_description.findAll("a")
-        if (m := search(r'(?:^[ ]*|furaffinity.net)/user/([\w.~-]+)', a.attrs.get("href"))) is not None
-    ))
+    mentions: List[str] = parse_mentions(tag_description)
+    folder: str = match(r"^/(scraps|gallery)/.*$", tag_folder["href"]).group(1).lower()
     file_url: str = "https:" + tag_file_url["href"]
 
     return {
@@ -149,5 +181,26 @@ def parse_submission_page(sub_page: BeautifulSoup) -> Dict[str, Union[int, str,
         "rating": rating,
         "description": description,
         "mentions": mentions,
+        "folder": folder,
         "file_url": file_url,
     }
+
+
+def parse_user_page(user_page: BeautifulSoup) -> Dict[str, str]:
+    tag_name: Tag = user_page.find("div", class_="username")
+    tag_profile: Tag = user_page.find("div", class_="userpage-profile")
+
+    status: str = (u := tag_name.find("span").text.strip())[0]
+    name: str = u[1:]
+    profile: str = "".join(map(str, tag_profile.children)).strip()
+
+    return {
+        "name": name,
+        "status": status,
+        "profile": profile,
+    }
+
+
+def parse_watchlist(watch_page: BeautifulSoup) -> List[Tuple[str, str]]:
+    tags_users: List[Tag] = watch_page.findAll("div", class_="watch-list-items")
+    return [((u := t.text.strip().replace(" ", ""))[0], u[1:]) for t in tags_users]
