@@ -2,18 +2,17 @@ from typing import List
 from typing import Optional
 
 from .parse import BeautifulSoup
-from .parse import ParsingError
 from .parse import Tag
-from .parse import check_page
+from .parse import check_page_raise
 from .parse import parse_submission_figure
 from .parse import parse_submission_page
 
 
 class SubmissionPartial:
-    def __init__(self, submission_figure: Optional[Tag] = None):
+    def __init__(self, submission_figure: Tag = None):
         assert submission_figure is None or isinstance(submission_figure, Tag)
 
-        self.submission_figure = submission_figure
+        self.submission_figure: Optional[Tag] = submission_figure
 
         self.id: int = 0
         self.title: str = ""
@@ -36,7 +35,9 @@ class SubmissionPartial:
     def __str__(self):
         return f"{self.id} {self.author} {self.title}"
 
-    def parse(self):
+    def parse(self, submission_figure: Tag = None):
+        assert submission_figure is None or isinstance(submission_figure, Tag)
+        self.submission_figure = submission_figure if submission_figure is not None else self.submission_figure
         if self.submission_figure is None:
             return
 
@@ -50,10 +51,10 @@ class SubmissionPartial:
 
 
 class Submission:
-    def __init__(self, submission_page: Optional[BeautifulSoup] = None):
+    def __init__(self, submission_page: BeautifulSoup = None):
         assert submission_page is None or isinstance(submission_page, BeautifulSoup)
 
-        self.submission_page = submission_page
+        self.submission_page: Optional[BeautifulSoup] = submission_page
 
         self.id: int = 0
         self.title: str = ""
@@ -66,6 +67,7 @@ class Submission:
         self.rating: str = ""
         self.description: str = ""
         self.mentions: List[str] = []
+        self.folder: str = ""
         self.file_url: str = ""
 
         self.parse()
@@ -82,6 +84,7 @@ class Submission:
         yield "rating", self.rating
         yield "description", self.description
         yield "mentions", self.mentions
+        yield "folder", self.folder
         yield "file_url", self.file_url
 
     def __repr__(self):
@@ -90,11 +93,13 @@ class Submission:
     def __str__(self):
         return f"{self.id} {self.author} {self.title}"
 
-    def parse(self):
+    def parse(self, submission_page: BeautifulSoup = None):
+        assert submission_page is None or isinstance(submission_page, BeautifulSoup)
+        self.submission_page = submission_page if submission_page is not None else self.submission_page
         if self.submission_page is None:
             return
-        elif (err := check_page(self.submission_page)) != 0:
-            raise ParsingError(f"Error: submission page error {err}")
+
+        check_page_raise(self.submission_page)
 
         parsed: dict = parse_submission_page(self.submission_page)
 
@@ -109,4 +114,5 @@ class Submission:
         self.rating: str = parsed["rating"]
         self.description: str = parsed["description"]
         self.mentions: List[str] = parsed["mentions"]
+        self.folder: str = parsed["folder"]
         self.file_url: str = parsed["file_url"]
